\ User Interface for AmForth FM Receiver for Arduino Uno
\ Written for AmForth
\ By: Craig A. Lindley
\ Last Update: 12/15/2013
\ Must have loadall.frt loaded

marker _UI_

\ Define ports and port bits
PORTD 5 portpin: POWERIND

&973 constant DEFAULT_CHANNEL
&1   constant DEFAULT_VOLUME

variable radioOn
variable volume

\ Create array in flash of 9 station/channel presets
create PRESETS
  0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 ,

\ Define PRESETS access functions
: ps!			( channel index --- )
    PRESETS + !i
;

: ps@			( index --- channel )
    PRESETS + @i
;

variable dps

\ Compile a string to flash and return its address and count
: f" 
    $22 parse 		( --- addr count )
    count ! addr !
    count @ 1 rshift count2 !
    count @ 2 mod 0 <> flag !

    dp dps !
    count2 @ 0
    do
        addr @ c@ 1 addr +!
        addr @ c@ 8 lshift or 1 addr +!
        ,
    loop
    flag @
    if
        addr @ c@ ,
    then
    dps @ count @
;
 
\ Define text that displays itself on the display
: text:			( col row addr count --- )
    create
    , , , ,
    does>
        1 clearRow
        dup over over
        3 + @i swap 2 + @i setCharPosition
        1+ @i swap @i iTypeString
;

\ Define the various text messages
4 1 f" Mute On"         text: MOMSG
4 1 f" Mute Off"        text: MOFFMSG
3 1 f" Volume Up"       text: VUMSG
2 1 f" Volume Down"     text: VDMSG
3 1 f" Seek Down"       text: SDMSG
4 1 f" Seek Up"         text: SUMSG
0 1 f" Preset Key 1..9" text: PKMSG
0 1 f" Preset Selected" text: PSSMSG

\ Format the channel for display
\ input: 973 output: FM - 97.3 MHz
: formatChannel		( channel --- address count )
    
    s>d <# [char] z hold [char] H hold 
           [char] M hold $20 hold # [char] . hold #s
           $20 hold [char] - hold $20 hold 
           [char] M hold [char] F hold
        #>
;

\ Display channel centered on first line of lcd
: displayChannel
    0 clearRow		\ Clear the row
    getChannel		( --- channel )
    formatChannel	( channel --- address count )
    dup
    16 swap - 2 / 0	\ Center the channel in row 0
    setCharPosition
    typeString
;

\ Run the FM radio
: runFMRadio

    \ Wait to allow hardware to stabilize
    500 ms

    0 radioOn !

    \ Initialize power indicator and turn off
    POWERIND pin_output
    POWERIND low

    \ Initialize the IR detector as input
    IRDETECTOR pin_input

    \ Init the LCD
    initLCD

    \ Init the FM radio
    initSI4703

    \ Set the default station
    DEFAULT_CHANNEL setChannel

    \ Set the default volume
    DEFAULT_VOLUME volume !

    begin
        \ Is a IR key code available ?
        iRCodeAvailable
        if				\ IR code available 
            decodeIRCode		\ See which key it is
            case
                IRC_ENTERSAVE		\ Preset assignment
                of
                    PKMSG		\ Prompt for preset key
                    waitForIRCode	\ Wait for key code
                        case
                            IRC_1KEY of getChannel 0 ps! PSSMSG endof
                            IRC_2KEY of getChannel 1 ps! PSSMSG endof
                            IRC_3KEY of getChannel 2 ps! PSSMSG endof
                            IRC_4KEY of getChannel 3 ps! PSSMSG endof
                            IRC_5KEY of getChannel 4 ps! PSSMSG endof
                            IRC_6KEY of getChannel 5 ps! PSSMSG endof
                            IRC_7KEY of getChannel 6 ps! PSSMSG endof
                            IRC_8KEY of getChannel 7 ps! PSSMSG endof
                            IRC_9KEY of getChannel 8 ps! PSSMSG endof
                        endcase
                endof

                IRC_VOL- 		\ Mute on
                of 
                    radioOn @
                    if 
                        1 muteControl
                        MOMSG
                    then 
                endof
                IRC_VOL+		\ Mute off
                of 
                    radioOn @
                    if 
                        0 muteControl
                        MOFFMSG
                    then 
                endof                
                IRC_PLAYPAUSE		\ Radio on/off
                of
                    radioOn @
                    if
                        0 radioOn !	\ Radio off
                        0 setVolume	\ Volume off
                        clearDisplay
                        0 displayMode   \ Display off
                        bl low		\ Backlight off
                        POWERIND low	\ Turn power indicator off
                    else
                        POWERIND high	\ Turn power indicator on
                        bl high		\ Backlight on
                        1 displayMode   \ Display on
                        DEFAULT_VOLUME setVolume
                        0 muteControl	\ Mute off
                        displayChannel	\ Show the channel
                        1 radioOn !	\ Radio on
                    then
                endof
                IRC_UPARROW 		\ Volume up
                of 
                    radioOn @
                    if 
                        1 volume +!
                        volume @ 15 >
                        if 15 volume ! then
                        volume @ setVolume
                        VUMSG
                    then 
                endof
                IRC_DOWNARROW		\ Volume down
                of 
                    radioOn @
                    if 
                        -1 volume +!
                        volume @ 1 <
                        if 1 volume ! then
                        volume @ setVolume
                        VDMSG
                    then 
                endof                

                IRC_RIGHTARROW 		\ Seek up
                of 
                    radioOn @
                    if 
                        1 seek
                        displayChannel
                        SUMSG
                    then 
                endof
                IRC_LEFTARROW		\ Seek down
                of 
                    radioOn @
                    if 
                        0 seek
                        displayChannel
                        SDMSG
                    then 
                endof                

                IRC_1KEY		\ Key 1 preset
                of 
                    radioOn @
                    if 
                        0 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                
                
                IRC_2KEY		\ Key 2 preset
                of 
                    radioOn @
                    if 
                        1 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                
                
                IRC_3KEY		\ Key 3 preset
                of 
                    radioOn @
                    if 
                        2 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_4KEY		\ Key 4 preset
                of 
                    radioOn @
                    if 
                        3 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_5KEY		\ Key 5 preset
                of 
                    radioOn @
                    if 
                        4 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_6KEY		\ Key 6 preset
                of 
                    radioOn @
                    if 
                        5 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_7KEY		\ Key 7 preset
                of 
                    radioOn @
                    if 
                        6 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_8KEY		\ Key 8 preset
                of 
                    radioOn @
                    if 
                        7 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

                IRC_9KEY		\ Key 9 preset
                of 
                    radioOn @
                    if 
                        8 ps@ dup
                        0 <>
                        if
                            setChannel 
                            displayChannel
                            PSSMSG
                        then
                    then 
                endof                

            endcase

        else				\ IR code unavailable
            radioOn @
            if
                updateReceptionIndicator
            else
                STEREOIND low
            then
        then
        0
    until
;

\ Prepare to run the FM radio as a turn key application

' runFMRadio is turnkey







